% Examples of ways to run multiple iterations in parallel.  (Also a serial example)

% Copyright 2019 The MathWorks, Inc.

figure(1);
clf;
set(gcf,'position',[15 60 980 420]);

fnNames = {'paramSweepSerial','paramSweepParfor','paramSweepParfeval',...
    'paramSweepSPMD','paramSweepSPMDadvanced','paramSweepCodistDrange',...
    'paramSweepPoollessParfor'};

thisProfile = 'kebnekaise';
poolSize = 3;

for ii = 1:length(fnNames)
    needsProfileInput = false;
    
    switch(fnNames{ii})
        case {'paramSweepSerial'}
            delete(gcp('nocreate')); % no pool
        case {'paramSweepPoollessParfor'}
            delete(gcp('nocreate')); % no pool
            needsProfileInput = true;
        otherwise
            if isempty(gcp('nocreate'))
                parpool(thisProfile,poolSize);
            end
    end
    
    subplot(2,4,ii);
    if needsProfileInput
        [Z,Zt] = feval(fnNames{ii}, linspace(100,150,6).', linspace(.5,2,6).', 1000, gca, poolSize, thisProfile);
    else
        [Z,Zt] = feval(fnNames{ii}, linspace(100,150,6).', linspace(.5,2,6).', 1000, gca);
    end
    title(fnNames{ii}); drawnow;
    showStats(fnNames{ii},Z,Zt);
    
end

function showStats(thisFn,Z,Zt)
Ztd = seconds(diff(Zt,[],3));

fprintf(1,'\n%s\n\tPeriod observed: %3.1f to %3.1f \n\tIteration times: %3.1f mean, %3.1f (std)\n',...
    thisFn, min(Z(:)), max(Z(:)), mean(Ztd(:)), std(Ztd(:)));
end
