function [Z,Zt,t0,N,M]=paramSweepParfeval(mu,nu,tMax,theseAxes) 
% [Z,Zt,t0,N,M]=paramSweepParfeval(mu,nu,tMax,theseAxes)
% parfeval code for parameter sweep study to investigate serial vs parallel
%
% Why use parfeval:
%    parfeval is robust to errors - one bad iteration will not crash entire job
%    parfeval allows you to add more work to the queue
%    parfeval allows you to collect some results and stop the rest of the queue
%
% Based on:
%    https://www.mathworks.com/help/distcomp/examples/...
%    plot-progress-during-parallel-computations-using-parfor-and-dataqueue.html

% Copyright 2019 The MathWorks, Inc

if nargin<4
    theseAxes=[];
end

%% Set up problem to provide a single index to loop on
[M,N] = meshgrid(mu,nu);
Z  = NaN(size(N));
Zt = NaT(size(N,1),size(N,2),2);

if ~isempty(theseAxes)
    visualizeParamSweep(M, N ,Z, theseAxes);
end


%% Parameter Sweep
t0 = getDatetime0;

p = gcp('nocreate');
% Asynchronous submission (Note: each iteration gets its own worker)
for ii = 1:numel(N)
    Zt(ii) = datetime; %start
    f(ii) = parfeval(p, @getVdpPeriod, 1, M(ii), N(ii), tMax); %#ok<AGROW>
end

% Synchronous retrieval (fetchNext will block until next result is retrieved)
for oo = 1:numel(N)
    [ii, Z(ii)] = fetchNext(f);
    Zt(ii+numel(Z)) = datetime; %stop

    updateSurface(theseAxes,[ii Z(ii)]);
end


end