function [Z,Zt,t0,N,M]=paramSweepParfor(mu,nu,tMax,theseAxes)
% [Z,Zt,t0,N,M]=paramSweepParfor(mu,nu,tMax,theseAxes)
% parfor code for parameter sweep study to investigate serial vs parallelParameter sweep study to investigate serial vs parallel
%
% Why use parfor:
%    parfor is the simplest method for parallelizing your work
%    parfor has infrastructure to passively balance load on each process
%    
%
% Based on:
%    https://www.mathworks.com/help/distcomp/examples/...
%    plot-progress-during-parallel-computations-using-parfor-and-dataqueue.html

% Copyright 2014-2019 The MathWorks, Inc.

if nargin<4
    theseAxes=[];
end

%% Set up problem to provide a single index to loop on
[M,N] = meshgrid(mu,nu);
Z  = NaN(size(N));
ZtIn = NaT(size(N));
ZtOut = NaT(size(N));
 
if ~isempty(theseAxes)
    visualizeParamSweep(M, N ,Z, theseAxes);
end

%% Parameter Sweep
t0 = getDatetime0;

D = parallel.pool.DataQueue;
D.afterEach(@(x) updateSurface(theseAxes,x));

parfor ii = 1:numel(N)
    ZtIn(ii) = datetime; %start
    Z(ii)= getVdpPeriod(M(ii), N(ii), tMax);
    ZtOut(ii) = datetime; %stop
    send(D, [ii Z(ii)]); 
end

if ~isempty(theseAxes)
    visualizeParamSweep(M, N ,Z, theseAxes);
    drawnow
end

Zt = cat(3,ZtIn,ZtOut);  

end