function [Z,Zt,t0,N,M]=paramSweepPoollessParfor(mu,nu,tMax,theseAxes,desiredTasks,thisCluster)
% [Z,Zt,t0,N,M]=paramSweepPoollessParfor(mu,nu,tMax,theseAxes,desiredTasks,thisCluster)
% parfor without a pool code for parameter sweep study to investigate serial vs parallel
%
% Why use parfor without a pool:
%    More robust for very long runs or runs with many workers (no MPI ring)
%    Prevents idle MATLAB processes while all iterations finish
%    Note: Does not pre-allocate resources - uses what is available
%    Note: Does not allow displays during run
%
% Based on:
%    https://www.mathworks.com/help/distcomp/examples/...
%    plot-progress-during-parallel-computations-using-parfor-and-dataqueue.html

% Copyright 2019 The MathWorks, Inc.

if nargin<5
    error('must input desiredTasks');
end

%% Set up problem to provide a single index to loop on
[M,N] = meshgrid(mu,nu);
Z  = NaN(size(N));
ZtIn = NaT(size(N));
ZtOut = NaT(size(N));
 
if ~isempty(theseAxes)
    visualizeParamSweep(M, N ,Z, theseAxes);
end

%% Set up opts
if nargin==6
    c = parcluster(thisCluster);
else
    c = getCurrentCluster();
    if isempty(c)
        error('Only valid with 5 or fewer inputs for batch or 6 inputs for interactive'); 
    end
end
 
opts = parforOptions(c, 'RangePartitionMethod', 'fixed', 'SubrangeSize', numel(N)/desiredTasks);

%% Parameter Sweep
t0=getDatetime0;

parfor(ii = 1:numel(N),opts)
    ZtIn(ii) = datetime; %start
    Z(ii) = getVdpPeriod(M(ii), N(ii), tMax);
    ZtOut(ii) = datetime; %stop
end

Zt = cat(3,ZtIn,ZtOut);

if ~isempty(theseAxes)
    visualizeParamSweep(M, N ,Z, theseAxes);
end

end