function [Z,Zt,t0,N,M]=paramSweepSPMD(mu,nu,tMax,theseAxes)
% [Z,Zt,t0,N,M]=paramSweepSPMD(mu,nu,tMax,theseAxes)
% spmd code for parameter sweep study to investigate serial vs parallel
%
% Why use spmd:
%    spmd allows you to pass information between workers
%    you can think of spmd as a way to access each worker and its environment 
%
% Based on:
%    https://www.mathworks.com/help/distcomp/examples/...
%    plot-progress-during-parallel-computations-using-parfor-and-dataqueue.html

% Copyright 2019 The MathWorks, Inc

if nargin<4
    theseAxes=[];
end

%% Set up problem to provide a single index to loop on
[M,N] = meshgrid(mu,nu);
Z  = NaN(size(N));

if ~isempty(theseAxes)
    visualizeParamSweep(M, N ,Z, theseAxes);
end

%% Parameter Sweep
t0 = getDatetime0;

D = parallel.pool.DataQueue;
D.afterEach(@(x) updateSurface(theseAxes,x));

p=gcp('nocreate');
if ~isempty(p)
    numWorkers = p.NumWorkers;
else
    error('paramSweepSPMD requires a pool of workers');
end

% Find indices for a for loop on each worker
divisions = linspace(0,numel(N),numWorkers+1).';
dividers = [1+divisions(1:end-1), divisions(2:end)];

spmd
    % Note: spmd only copies variables referenced inside the spmd block
    idx = dividers(labindex,1):dividers(labindex,2);
    
    % Make local variables that can be re-assembled with cat across workers
    Zw = NaN(length(idx),1);
    Ztw = NaT(length(idx),2);
    
    for ii = 1:length(idx)
        Ztw(ii,1) = datetime;   %start
        Zw(ii) = getVdpPeriod(M(idx(ii)), N(idx(ii)), tMax);
        Ztw(ii,2) = datetime; %stop
        
        send(D, [idx(ii) Zw(ii)]);
    end
end

% Reassemble with cat across the sliced dimension
Z = reshape(cat(1,Zw{:}),size(N));
Zt = reshape(cat(1,Ztw{:}),size(N,1),size(N,2),2);

end