function [Z,Zt,t0,N,M]=paramSweepSerial(mu,nu,tMax,theseAxes)
% [Z,Zt,t0,N,M]=paramSweepSerial(mu,nu,tMax,theseAxes)
% Serial code for parameter sweep study to investigate serial vs parallel
%
% Based on:
%    https://www.mathworks.com/help/distcomp/examples/...
%    plot-progress-during-parallel-computations-using-parfor-and-dataqueue.html

% Copyright 2014-2019 The MathWorks, Inc.

if nargin<4
    theseAxes=[];
end

%% Set up problem to provide a single index to loop on
[M,N] = meshgrid(mu,nu);
Z  = NaN(size(N));
Zt = NaT(size(N,1),size(N,2),2);

if ~isempty(theseAxes)
    visualizeParamSweep(M,N, Z, theseAxes);
end

%% Parameter Sweep
t0 = getDatetime0;

for ii = 1:numel(N)
    Zt(ii) = datetime;   %start
    Z(ii) = getVdpPeriod(M(ii), N(ii), tMax);
    Zt(ii+numel(Z)) = datetime; %stop
    updateSurface(theseAxes,[ii Z(ii)]);
end

end