#1. Analyzing Patient Data
#read.csv(file = "data/inflammation-01.csv")
dat <- read.csv(file = "data/inflammation-01.csv", header = FALSE)
head(dat)

?read.csv

# first value in dat, row 1, column 1
dat[1, 1]
# middle value in dat, row 30, column 20
dat[30, 20]
# pick columns 10 and 20 from rows 1, 3, and 5
dat[c(1, 3, 5), c(10, 20)]
# select the first ten columns of values for the first four rows
dat[1:4, 1:10]

#Addressing columns by name
dat$V16

# mean inflammation on day 7
mean(dat[, 7])
# standard deviation of inflammation on day 7
sd(dat[, 7])

# Summarize function
summary(dat[, 1:4])

#Compute the mean over the rows
avg_patient_inflammation <- apply(dat, 1, mean)
#Compute the mean over the columns
avg_day_inflammation <- apply(dat, 2, mean)
#Alternatives for this rowMeans and colMeans

#Subsetting data
animal <- c("m", "o", "n", "k", "e", "y")
# first three characters
animal[1:3]
# delete the first column
animal[-1]

#calculate the mean inflammation for every second day (across all patients)
apply(dat[, seq(1, 40, by = 2)], 2, mean)

#Plotting
avg_day_inflammation <- apply(dat, 2, mean)
plot(avg_day_inflammation)
max_day_inflammation <- apply(dat, 2, max)
plot(max_day_inflammation)
min_day_inflammation <- apply(dat, 2, min)
plot(min_day_inflammation)

#2. Creating Functions
fahrenheit_to_kelvin <- function(temp_F) {
  temp_K <- ((temp_F - 32) * (5 / 9)) + 273.15
  return(temp_K)
}
fahrenheit_to_kelvin(32)

#Composing functions
kelvin_to_celsius <- function(temp_K) {
  temp_C <- temp_K - 273.15
  return(temp_C)
}

# absolute zero in Celsius
kelvin_to_celsius(0)

fahrenheit_to_celsius <- function(temp_F) {
  temp_K <- fahrenheit_to_kelvin(temp_F)
  temp_C <- kelvin_to_celsius(temp_K)
  return(temp_C)
}

# freezing point of water in Celsius
fahrenheit_to_celsius(32.0)

#Nesting functions
# freezing point of water in Celsius
kelvin_to_celsius(fahrenheit_to_kelvin(32.0))


#Analyzing Multiple Data Sets
analyze <- function(filename) {
  # Plots the average, min, and max inflammation over time.
  # Input is character string of a csv file.
  dat <- read.csv(file = filename, header = FALSE)
  avg_day_inflammation <- apply(dat, 2, mean)
  plot(avg_day_inflammation)
  max_day_inflammation <- apply(dat, 2, max)
  plot(max_day_inflammation)
  min_day_inflammation <- apply(dat, 2, min)
  plot(min_day_inflammation)
}

analyze("data/inflammation-01.csv")

#For Loops
best_practice <- c("Let", "the", "computer", "do", "the", "work")
print_words <- function(sentence) {
  print(sentence[1])
  print(sentence[2])
  print(sentence[3])
  print(sentence[4])
  print(sentence[5])
  print(sentence[6])
}

print_words(best_practice)

print_words <- function(sentence) {
  for (word in sentence) {
    print(word)
  }
}

print_words(best_practice)

len <- 0
vowels <- c("a", "e", "i", "o", "u")
for (v in vowels) {
  len <- len + 1
}
# Number of vowels
len

letter <- "z"
for (letter in c("a", "b", "c")) {
  print(letter)
}

list.files(path = "data", pattern = "csv")
list.files(path = "data", pattern = "inflammation")
list.files(path = "data", pattern = "csv", full.names = TRUE)



filenames <- list.files(path = "data",  
                        # Now follows a regular expression that matches:
                        pattern = "inflammation-[0-9]{2}.csv",
                        #          |            |        the standard file extension of comma-separated values
                        #          |            the variable parts (two digits, each between 0 and 9)
                        #          the static part of the filenames
                        full.names = TRUE)
filenames <- filenames[1:3]
for (f in filenames) {
  print(f)
  analyze(f)
}

analyze_all <- function(pattern) {
  # Runs the function analyze for each file in the current working directory
  # that contains the given pattern.
  filenames <- list.files(path = "data", pattern = pattern, full.names = TRUE)
  for (f in filenames) {
    analyze(f)
  }
}


#Making Choices
analyze <- function(filename) {
  # Plots the average, min, and max inflammation over time.
  # Input is character string of a csv file.
  dat <- read.csv(file = filename, header = FALSE)
  avg_day_inflammation <- apply(dat, 2, mean)
  plot(avg_day_inflammation)
  max_day_inflammation <- apply(dat, 2, max)
  plot(max_day_inflammation)
  min_day_inflammation <- apply(dat, 2, min)
  plot(min_day_inflammation)
}
analyze_all <- function(pattern) {
  # Runs the function analyze for each file in the current working directory
  # that contains the given pattern.
  filenames <- list.files(path = "data", pattern = pattern, full.names = TRUE)
  for (f in filenames) {
    analyze(f)
  }
}

pdf("inflammation-01.pdf")
analyze("data/inflammation-01.csv")
dev.off()

#Conditionals
num <- 37
if (num > 100) {
  print("greater")
} else {
  print("not greater")
}
print("done")

sign <- function(num) {
  if (num > 0) {
    return(1)
  } else if (num == 0) {
    return(0)
  } else {
    return(-1)
  }
}

sign(-3)

dat <- read.csv("data/inflammation-01.csv", header = FALSE)
plot_dist <- function(x, threshold) {
  if (length(x) > threshold) {
    boxplot(x)
  } else {
    stripchart(x)
  }
}
plot_dist(dat[, 10], threshold = 10)     # day (column) 10


#Command-line Programs




















